#!/usr/bin/python3 -u

import sys
from .actions import Action
from py_console import console

def get_action_methods():
    method_list = [method for method in dir(Action) if method.startswith('__') is False]
    return method_list
    
def main():
    """
    Everything we want to wrap before we release the command
    and its child argumentes to terraform cli. If the first sys
    argument is not mapped, we release the call to terraform.
    """

    get_action_methods()
    if len(sys.argv) == 1 or not filter(sys.argv[1].startswith, get_action_methods()):
        arg = "help"
        target_environment = None
    elif "-" in sys.argv[1]:
        arg = sys.argv[1].split('-')[0]
        target_environment = sys.argv[1].split('-')[1].lower()
    else:
        arg = sys.argv[1]
        target_environment = None

    try:
        current_action = Action(arg, target_environment)
        func = getattr(current_action, arg)
        func()
    except KeyboardInterrupt:
        console.success("\n Execution terminated", showTime=False)
        sys.exit(130)
    except(ValueError):
        pass   



#!/usr/bin/python3 -u

from .core import Core
from .workspace import Workspace
from py_console import console
from importlib.metadata import version, PackageNotFoundError
import json
import jsonpickle
import os
import pkg_resources
import sys
import shutil, stat
import subprocess

class Action(Core):
    """
    Inherits the Base Class and its attributes.
    Adding in 2 child atts, action, and target_environment.
    """
    def __init__(self, action, target_environment):
        self.action = action
        self.target_environment = target_environment
        Core.__init__(self, action, target_environment)

    def __str__(self):
        return ' '.join((self.repo_name, self.location))

    def command(self, command):
        """
        Re-usable function to execute a shell command, 
        with error handling, and ability to execute quietly, 
        or display output.
        """
        output = subprocess.Popen(command, env=self.my_env)
        output.communicate()
        
    def apply(self):
        self.init()
        console.success("  Running Terraform Apply", showTime=False)
        apply = ['terraform', 'apply'] + self.var_file_args + sys.argv[2:]
        self.command(apply) 

    def applynoprompt(self):
        self.reinit()
        console.success("  Running Terraform Apply", showTime=False)
        applynoprompt = ['terraform', 'apply', '-input=false', '-auto-approve'] + self.var_file_args + sys.argv[2:]
        self.command(applynoprompt) 

    def config(self):
        import getopt, yaml
        options_list = [item + '=' for item in list(self.options_dict.keys())]
        config_file_name = self.user_config_path

        try:
            opts, args = getopt.getopt(sys.argv[2:], "", options_list)
        except getopt.GetoptError as errorname:
            print('Error: ' + str(errorname))
            sys.exit(2)

        for o, a in opts:
            option = o.replace('-', '')

            if os.path.exists(config_file_name):
                with open(config_file_name) as f:
                    doc = yaml.safe_load(f)
                doc[option] = a
                with open(config_file_name, 'w') as f:
                    yaml.safe_dump(doc, f, default_flow_style=False)
            else:
                dict = {option: a}
                with open(config_file_name, 'w') as f:
                    yaml.safe_dump(dict, f, default_flow_style=False)

        console.success("\n Config Path:\n   " + config_file_name, showTime=False)
        console.success(" Current Config:", showTime=False)
        config = open(config_file_name, "r")
        config_content = config.read().splitlines()
        for line in config_content:
            console.success("  " + line, showTime=False)


    def destroy(self):
        """
        Force Destroy on Terraform, based on site, and resources.
        """
        self.init()
        console.success("  Running Terraform Destroy", showTime=False)
        destroy = ['terraform', 'destroy'] + self.var_file_args + sys.argv[2:]
        self.command(destroy) 

    def destroyforce(self):
        """
        Force Destroy on Terraform, based on site, and resources.
        """
        self.reinit()
        console.success("  Running Terraform Destroy Force", showTime=False)
        destroyforce = ['terraform', 'destroy', '-force'] + self.var_file_args + sys.argv[2:]
        self.command(destroyforce) 

    def help(self):
        """
        Provide Application Help
        """
        help = """
        Usage:
           {0} <command>
           {0} <command>-<site>

        Example:
           {0} plan
           {0} plan-dr
           {0} config --bucket_prefix=test_bucket --tf_cloud_org=test_org

        Commands:
           apply          Apply Terraform Configuration
           config         Configure {0} deployment global variables
           destroy        Destroy Terraform Configuration
           destroyforce   Destroy Terraform Configuration with no prompt
           help           Display the help menu that shows available commands
           init           Initialize Terraform backend and clean local cache
           plan           Create Terraform plan with clean local cache
           plandestroy    Create a Plan for a Destroy scenario
           reinit         Initialize Terraform backend and keep local cache
           replan         Create Terraform plan with existing local cache
           taint          Taint specific module and resources
           test           Test run showing all project variables
           tfimport       Import states for existing resources
           update         Update Terraform modules
           version        App version
        """
        print(help.format(self.app_name))

    def init(self):
        """
        Initialize the terraform backend using the appropriate env,
        and variables.
        """
        console.success("  Initializing Terraform", showTime=False)
        cleanup_list = ['.terraform.lock.hcl', '.terraform']
        for item in cleanup_list:
            if os.path.exists(item):
                console.success("  Removing " + item, showTime=False)
                def del_rw(action, name, exc):
                    os.chmod(name, stat.S_IWRITE)
                    os.remove(name)
                if os.path.isfile(item):
                    os.remove(item)
                else:
                    shutil.rmtree(item, onerror=del_rw)
        self.reinit()

    def plan(self):
        self.init()
        console.success("  Creating a Terraform Plan", showTime=False)
        plan = ['terraform', 'plan'] + self.var_file_args + sys.argv[2:]
        self.command(plan) 

    def plandestroy(self):
        self.init()
        console.success("  Creating a Destroy Plan", showTime=False)
        plandestroy = ['terraform', 'plan', '-input=false', '-refresh=true', '-destroy'] + self.var_file_args + sys.argv[2:]
        self.command(plandestroy) 

    def refresh(self):
        console.success("  Running Terraform Refresh", showTime=False)
        refresh = ['terraform', 'refresh'] + self.var_file_args + sys.argv[2:]
        self.command(refresh) 

    def reinit(self):
        """
        Initialize the terraform backend using the appropriate env,
        and variables.
        """
        if self.backend_type == "aws":
            console.success("  Initializing AWS Backend", showTime=False)
            self.command(
                [
                    'terraform', 'init',
                    '-backend-config', 'region='+self.backend_region,
                    '-backend-config', 'bucket='+self.bucket,
                    '-backend-config', 'key='+self.bucket_key
                    ]
                ) 
        elif self.backend_type == "azr":
            console.success("  Initializing AZR Backend", showTime=False)
            self.command(
                [
                    'terraform', 'init',
                    '-backend-config', 'storage_account_name='+self.bucket, 
                    '-backend-config', 'container_name='+self.account+'1', 
                    '-backend-config', 'key='+self.bucket_key, 
                    '-backend-config', self.secret_path
                    ]
                )
        elif self.backend_type == "tfc" or self.tf_cloud_backend == "true":
            console.success("  Initializing Terraform Cloud Backend", showTime=False)
            Workspace(self.bucket_key, self.platform, self.version_tf(), self.tf_cloud_backend_org)
            if not os.path.exists('.terraform'):
                console.success("  Creating .terraform directory and backend configuration", showTime=False)
                os.makedirs('.terraform')
                backend_config = '.terraform/backend-'+self.environment+'.hcl'
                with open(backend_config, 'w') as config_object:
                    config_object.write('workspaces { name = \"'+self.bucket_key+'" }')
            self.command(
                [
                    'terraform', 'init',
                    '-backend-config', 'organization='+self.account, 
                    '-backend-config', '.terraform/backend-'+self.environment+'.hcl'
                    ]
                )
        else:
            console.success("  Initializing Terraform Local Backend", showTime=False)
            self.command(
                [
                    'terraform', 'init'
                    ]
                )

    def replan(self):
        self.reinit()
        console.success("  Running Terraform Plan", showTime=False)
        plan = ['terraform', 'plan'] + self.var_file_args + sys.argv[2:]
        self.command(plan) 

    def taintresources(self):
        """
        Create Taint resources list.
        """
        console.success("  Running Terraform Resource Query", showTime=False)
        modules = []
        resources = []
        show_cmd = subprocess.Popen(['terraform', 'show'],stdin=None,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        (stdout, stderr) = show_cmd.communicate()
        lines = stdout.strip().decode().replace(":","").splitlines()
        for line in lines:
            if "module" in line:
                items = line.split(".")
                if len(items) > 1:
                    module = items[1]
                    if module not in modules:
                        modules.append(module)
       
        if len(modules) > 1:
            i = 0
            print("Please choose the module you would like to taint:")
            for module in modules:
                print('[', i, ']: ', module)
                i += 1
            selectedmoduleindex = input("Module Selection: ")
            if int(selectedmoduleindex) > (len(modules) - 1):
                print('You selected an invalid module index, please try again')
                sys.exit(0)
        
            selected_module = modules[int(selectedmoduleindex)]
            print(selected_module)
        
        for line in lines:
            if selected_module in line:
                items = line.split(".")
                if len(items) > 1:
                    resource = items[2]
                    if resource not in resources:
                        resources.append(resource)
    
        if len(resources) > 1:
            i = 0
            print("Please choose the resource you would like to taint:")
            for resource in resources:
                print('[', i, ']: ', resource)
                i += 1
            selectedresourceindex = input("Resource Selection: ")
            if int(selectedresourceindex) > (len(resources) - 1):
                print('You selected an invalid resource index, please try again')
                sys.exit(0)
        
            selected_resource = resources[int(selectedresourceindex)]
            print(selected_resource)
            return selected_module, selected_resource

    def taint(self):
        """
        Taint resources.
        """
        self.init()
        modules_totaint = []
        resources_totaint = []
        console.success("  Running Terraform Taint", showTime=False)
        taintagain = 'y'

        while taintagain == 'y':
            taintresources = self.taintresources()
            modules_totaint.append(taintresources[0])
            resources_totaint.append(taintresources[1])
            taintagain = input("Select another module to taint? [y/n] : ")

        print('Tainting the following resources:')
        if len(modules_totaint) >= 1:
            i = 0
            for i in range(len(modules_totaint)):
                print('[', i, ']: [', modules_totaint[i], '] ', resources_totaint[i])
                i += 1

        gotaint = input("Proceed with Taint? [y/n] : ")
        
        if gotaint == 'y':
            if len(modules_totaint) >= 1:
                i = 0
                for i in range(len(modules_totaint)):
                    taint = ['terraform', 'taint'] + self.var_file_args + ['-module='+modules_totaint[i], resources_totaint[i]] + sys.argv[2:]
                    self.command(taint) 
                    i += 1

    def test(self):
        """
        Test all class attributes through a single cli call. 
        """
        console.warn("  Terraform Prerequisites Check", showTime=False)
        console.warn("  =============================", showTime=False)
        self.version_git()
        self.version_tf()

        serialized = json.loads(jsonpickle.encode(self, max_depth=2))

        console.warn("\n  Current Deployment Details", showTime=False)
        console.warn("  ==========================", showTime=False)
        console.success("  Platform           = {platform}".format(**serialized), showTime=False)
        console.success("  AppDir             = {location}".format(**serialized), showTime=False)
        console.success("  Repo Name          = {repo_name}".format(**serialized), showTime=False)
        console.success("  Repo Root          = {repo_root}".format(**serialized), showTime=False)
        console.success("  Repo URL           = {repo_url}".format(**serialized), showTime=False)
        console.success("  Repo Preefix       = {repo_prefix}".format(**serialized), showTime=False)
        console.success("  Cloud              = {cloud}".format(**serialized), showTime=False)
        console.success("  Project            = {project}".format(**serialized), showTime=False)
        console.success("  Branch Name        = {branch_name}".format(**serialized), showTime=False)
        console.success("  Resource Name      = {resource}".format(**serialized), showTime=False)
        console.success("  Account            = {account}".format(**serialized), showTime=False)
        console.success("  Environment        = {environment}".format(**serialized), showTime=False)
        console.success("  Common Shell File  = {common_shell_file}".format(**serialized), showTime=False)
        console.success("  Common Env File    = {common_env_file}".format(**serialized), showTime=False)
        console.success("  Local Env File     = {local_env_file}".format(**serialized), showTime=False)
        console.success("  Site (Target Env.) = {site}".format(**serialized), showTime=False)
        console.success("  Command            = {action}".format(**serialized), showTime=False)
        console.success("  DR                 = {dr}".format(**serialized), showTime=False)
        console.success("  Prefix             = {prefix}".format(**serialized), showTime=False)
        console.success("  Module             = {module}".format(**serialized), showTime=False)
        console.success("  Backend Secret     = {secret_path}".format(**serialized), showTime=False)
        console.success("  Deployment Region  = {region}".format(**serialized), showTime=False)
        console.success("  Backend Region     = {backend_region}".format(**serialized), showTime=False)
        console.success("  Bucket             = {bucket}".format(**serialized), showTime=False)
        console.success("  Key                = {bucket_key}".format(**serialized), showTime=False)
        console.success("  Backend Type       = {backend_type}".format(**serialized), showTime=False)
        console.success("  TF Cloud Backend   = {tf_cloud_backend}".format(**serialized), showTime=False)
        console.success("  China Deployment   = {china_deployment}".format(**serialized), showTime=False)

        console.warn("\n  Terraform Variables", showTime=False)
        console.warn("  ===================", showTime=False)
        console.success("  TF_VAR_mode              = {mode}".format(**serialized), showTime=False)
        console.success("  TF_VAR_project           = {project}".format(**serialized), showTime=False)
        console.success("  TF_VAR_prefix            = {prefix}".format(**serialized), showTime=False)
        console.success("  TF_VAR_account           = {account}".format(**serialized), showTime=False)
        console.success("  TF_VAR_env               = {environment}".format(**serialized), showTime=False)
        console.success("  TF_VAR_site              = {site}".format(**serialized), showTime=False)
        console.success("  TF_VAR_deployment_region = {region}".format(**serialized), showTime=False)
        console.success("  TF_VAR_backend_region    = {backend_region}".format(**serialized), showTime=False)
        console.success("  TF_VAR_bucket            = {bucket}".format(**serialized), showTime=False)
        console.success("  TF_VAR_azrsa             = {bucket}".format(**serialized), showTime=False)
        console.success("  TF_CLI_ARGS              = {tf_cli_args}".format(**serialized), showTime=False)
        console.success("  TF_VAR_china_deployment  = {china_deployment}".format(**serialized), showTime=False)

        console.warn("\n  Global Config Variables", showTime=False)
        console.warn("  =======================", showTime=False)
        console.success("  bucket_prefix           = {bucket_prefix}".format(**serialized), showTime=False)
        console.success("  tf_cloud_org            = {tf_cloud_backend_org}".format(**serialized), showTime=False)

    def tfimport(self):
        self.init()
        console.success("  Running Terraform Import", showTime=False)
        tfimport = ['terraform', 'import'] + self.var_file_args + sys.argv[2:]
        self.command(tfimport) 

    def update(self):
        console.success("  Updating Modules", showTime=False)
        update = ['terraform', 'get', '-update=true'] + self.var_file_args + sys.argv[2:]
        self.command(update) 

    def validate(self):
        console.success("  Running Terraform Validation", showTime=False)
        self.command(['terraform', 'init', '-backend=false']) 
        validate = ['terraform', 'validate'] + sys.argv[2:]
        self.command(validate) 

    def version(self):
        """
        Get application version from VERSION with cli call.
        """
        console.success("  " + self.app_config.upper() + " version: " + version("tfbuild"), showTime=False)

    def version_git(self):
        """
        Get the version of Git used.
        """
        try:    
            show_cmd = subprocess.Popen(['git', '--version'],stdin=None,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
            (stdout, stderr) = show_cmd.communicate()
            git_version = stdout.strip().decode().split(" ")[2]
            console.success("  Git version: " + git_version, showTime=False)
        except:
            console.error("  Git is not installed", showTime=False)
            sys.exit(2)  

        return git_version

    def version_tf(self):
        """
        Get the version of Terraform used.
        """
        try:
            show_cmd = subprocess.Popen(['terraform', 'version', '-json'],stdin=None,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
            (stdout, stderr) = show_cmd.communicate()
            output = json.loads(stdout)
            terraform_version = output['terraform_version']
            console.success("  Terraform version: " + terraform_version, showTime=False)
        except:
            console.error("  Terraform is not installed", showTime=False)
            sys.exit(2)  

        return terraform_version




#!/usr/bin/python3 -u

from py_console import console
import hcl
import json
import os
import requests
import sys

class Workspace(object):
    def __init__(self, tf_workspace_name, platform, tf_version, org):
        self.platform = platform
        self.base_url = 'https://app.terraform.io'
        self.org = org
        self.get_org()
        self.get_terraformrc_path()
        self.org_url = self.base_url + '/api/v2/organizations/' + self.org
        self.workspaces_url = self.org_url + '/workspaces'
        self.token_environment_variable_name = "TF_TOKEN"
        self.tfc_config_template = {
            "credentials": {
                "app.terraform.io": {
                    "token": "temp_value"
                }
            },
            "disable_checkpoint": "true",
            "plugin_cache_dir": self.cache_dir
        }
        self.tf_headers = {
            "Content-Type": "application/vnd.api+json",
            "Authorization": "Bearer {}".format(self.get_token()),
        }
        self.create_workspace(tf_workspace_name, tf_version)

    def get_org(self):
        if not self.org:
            console.error("  Please add 'tf_cloud_org' to the local config file !\n", showTime=False)
            sys.exit(2)

    def get_terraformrc_path(self):
        if self.platform == 'windows':
            self.config_file = os.path.join(os.path.expanduser("~"), "AppData", "Roaming", "terraform.rc")
            self.cache_dir = os.path.join(os.path.expanduser("~"), "AppData", "Roaming", "terraform.d", "plugin-cache")
        else:
            self.config_file = os.path.join(os.path.expanduser("~"), ".terraformrc")
            self.cache_dir = os.path.join(os.path.expanduser("~"), "terraform.d", "plugin-cache")

    def get_token(self):
        if os.environ.get(self.token_environment_variable_name) is not None:
            token = os.environ[self.token_environment_variable_name]
            if (os.path.exists(self.config_file)) and (os.stat(self.config_file).st_size != 0):
                with open(self.config_file, 'r') as cf:
                    cf_obj = json.load(cf)
                    cf_obj['credentials']['app.terraform.io']['token'] = token
                with open(self.config_file, 'w') as cf:
                    json.dump(cf_obj, cf, indent = 4, sort_keys=True)
            else:
                console.warn("\n  Config file: " + self.config_file + " is being configured from env variable !", showTime=False)
                with open(self.config_file, 'w') as cf:
                    json_cf = json.loads(json.dumps(self.tfc_config_template))
                    json_cf['credentials']['app.terraform.io']['token'] = token
                    json.dump(json_cf, cf, indent = 4, sort_keys=True)
        elif os.path.exists(self.config_file):
            if os.stat(self.config_file).st_size == 0:
                console.error("  The TFC config file: " + self.config_file + " is empty !", showTime=False)
                sys.exit(2)
            else:
                with open(self.config_file, 'r') as fp:
                    obj = hcl.load(fp)
                    token = obj['credentials']['app.terraform.io']['token']
        else:
            console.error("  Please configure the 'TF_TOKEN' env variable or \n  the '" + self.config_file + "' file \n  to contain the deployment token !\n", showTime=False)
            sys.exit(2)
        return token

    def get_workspaces(self):
        workspace_list = []
        read_workspaces = requests.get(self.workspaces_url, headers = self.tf_headers)
        r_json = read_workspaces.json()
        if read_workspaces.ok:
            for item in r_json['data']:
                workspace_name = item['attributes']['name']
                workspace_list.append(workspace_name)
        else:
            console.error("\n  Your 'tf_cloud_org' variable or your credentials are invalid !", showTime=False)
            console.error("  - Error Code: " + str(read_workspaces.status_code) + "\n  - Error Message: " + r_json['errors'][0]['title'], showTime=False)
            sys.exit(2)
        return workspace_list

    def create_workspace(self, name, tf_version):
        workspace_data = dict(
            attributes={
                'name':name,
                'terraform-version':tf_version,
                'execution-mode':'local'
            }
        )
        workspace_data = dict(
            data=workspace_data
        )
        
        if name not in self.get_workspaces():
            requests.post(self.workspaces_url, headers=self.tf_headers, data=json.dumps(workspace_data))



#!/usr/bin/python3 -u

from git import Repo
from py_console import console
import confuse
import hcl
import os
import sys

class Core():
    def __init__(self, action, target_environment=None):
        self.app_name = os.path.basename(sys.argv[0])
        self.app_config = os.path.basename(os.path.dirname(__file__))
        self.action = action
        if self.action != "help":
            self.get_platform()
            self.build_id = os.getenv('BUILD_ID')
            self.target_environment = target_environment
            self.location = os.path.realpath(os.getcwd())
            self.repo_name = self.get_repo_name(self.repo_url)
            self.branch_name = str(Repo(self.repo_root).active_branch).lower()
            self.clouds_list = ['aws', 'azr', 'vmw', 'gcp']
            self.global_resources = ["53", "global"]
            self.get_default_variables()
            self.options_dict = {
                "bucket_prefix": "inf.tfstate", 
                "tf_cloud_org": None
                }
            self.bucket_prefix = self.set_config_var('bucket_prefix')
            self.tf_cloud_org1 =  self.set_config_var('tf_cloud_org')
            self.user_config_path = self.load_configs()[1]
            self.resource = os.path.relpath(self.location, self.repo_root).replace('\\', '/')
            self.config_files = self.load_configs()
            self.secret_path = os.path.join("{}".format(self.repo_root), "secret_{}_backend.tfvars".format(self.cloud))
            self.get_env_files()
            self.get_deployment_attributes()
            self.sanity_check()
            self.set_site_configuration()
            self.set_backend_configuration(self.backend.lower())
            self.export_environment()

    def get_platform(self):
        try:
            repo_root = Repo(search_parent_directories=True).git.rev_parse("--show-toplevel")
        except:
            console.error("  You are not executing " + self.app_config.upper() + " from a git repository !\n  Please ensure execution from a resurce directory inside a git repository !\n", showTime=False)
            sys.exit(2)

        if sys.platform.startswith("win"):
            self.platform = "windows"
            self.repo_root = repo_root.replace('/', '\\')
        else:
            self.platform = "linux"
            self.repo_root = repo_root

        try:
            self.repo_url = Repo(self.repo_root).remotes[0].config_reader.get("url")
        except IndexError:
            console.error("  " + str(os.path.splitext(os.path.basename(self.repo_root))[0]).upper() + " is a local repository with no remotes. !\n", showTime=False)
            sys.exit(2)

    def get_repo_name(self, repo_url: str) -> str:
        """
        Extract the repository name from various Git remote URL formats.
        """

        for prefix in ["codecommit::", "codecommit://", "https://", "http://", "git@"]:
            if repo_url.startswith(prefix):
                repo_url = repo_url[len(prefix):]

        repo_url = repo_url.replace(":", "/").replace("@", "/")
        repo_name = os.path.splitext(os.path.basename(repo_url))[0]
        return repo_name.lower()

    def load_configs(self):
        config = confuse.LazyConfig(self.app_config, __name__)
        user_config_path = config.user_config_path()
        if os.path.isfile(user_config_path):
            config.set_file(user_config_path, base_for_paths=True)
        return config, user_config_path

    def set_config_var(self, var):
        if os.environ.get(var.upper()) is not None:
            env_var = os.environ[var.upper()]
        else:
            env_var = self.load_configs()[0][var].get(confuse.Optional(str, default=self.options_dict[var]))
        return env_var


    def get_default_variables(self):
        """
        Get Repository Prefix, Cloud Dependent Project, Account, Environment variables.
        """

        self.repo_name_parts = self.repo_name.split("-")

        if len(self.repo_name_parts) > 2:
            self.repo_prefix = "-".join(self.repo_name_parts[:-2])
        else:
            self.repo_prefix = ""

        if len(self.repo_name_parts) <= 1:
            console.error(
                "Error: Invalid repository name structure.\n"
                "The repository name structure needs to be:\n"
                "  [Repository Prefix]-[Hosting Platform]-[Project name]\n"
                "  Note: Project Prefix is optional.\n"
                "  Example:\n"
                "    - myrepo-aws-myproject\n"
                "    - aws-myproject\n"
                "Default supported Hosting Platform values are:\n  " + ", ".join(self.clouds_list) + "\n",
                showTime=False
            )
            sys.exit(2)
        else:
            self.cloud = self.repo_name_parts[-2]

        if self.cloud in self.clouds_list:
            self.project = self.repo_name_parts[-1]
            self.account = self.branch_name.split("-")[0]
            if len(self.branch_name.split("-")) == 2: 
                self.environment = self.branch_name.split("-")[1]
            else:
                console.error("  Error: Invalid repository branch name structure.\n"
                              "  For Cloud Hosting Platforms, he branch name structure needs to be:\n"
                              "  [Account Number]-[Environment Name]\n"
                              "  Example:\n    - 31234565435-uat\n", showTime=False)
                sys.exit(2)
        else:
            self.project = self.repo_name.split("-")[-1]
            if len(self.branch_name.split("-")) == 2:
                self.account = self.branch_name.split("-")[0]
                self.environment = self.branch_name.split("-")[1]
            else:
                self.account = 'none'
                self.environment = self.branch_name

    def get_env_files(self):
        """
        Return Appropriate Env File Based on whether there is
        a target deployment defined in the init attributes.
        """
        if self.target_environment:
            file_preffix = "{}_{}".format(self.environment, self.target_environment)
        else:
            file_preffix = "{}".format(self.environment)
        
        self.common_shell_file = os.path.join(
            self.repo_root, "common", "environments","env_{}.hcl".format(file_preffix))
        self.common_env_file = os.path.join(
            self.repo_root, "common", "environments","env_{}_common.tfvars".format(file_preffix))
        self.local_env_file = os.path.join(
            self.location, "environments", "env_{}.tfvars".format(file_preffix))

    def get_deployment_attributes(self):
        """
        Extract deployment attributes from shell env file.
        This is done to maintain backward compatibility with
        current deployments, but should be migrated to a declarative
        language in the future, ie: json,yaml.
        """

        if not os.path.isfile(self.common_shell_file):
            console.error("  No Common Wrapper Shell File available ! Please create:\n  " + self.common_shell_file + "\n  and add configuration content if necessary !\n", showTime=False)
            sys.exit(2)

        try:    
            with open(self.common_shell_file, 'r') as fp:
                obj = hcl.load(fp)
                self.china_deployment = obj.get('china_deployment', '').lower()
                self.dr = obj.get('dr', '').lower()
                self.global_resource = obj.get('global_resource', '').lower()
                self.target_environment_type = obj.get('target_environment_type', 'region').lower()
                self.mode = obj.get('mode', '').lower()
                self.region = obj.get('region', '').lower()
                self.backend = obj.get('backend', '').lower()
                self.tf_cloud_backend = obj.get('tf_cloud_backend', 'simple')   .lower()            
                self.tf_cloud_org2 = obj.get('tf_cloud_org', '').lower()               
                if sys.platform.startswith("win"):
                    self.tf_cli_args = obj.get('tf_cli_args', '').replace('"','').replace('${REPO_PATH}',self.repo_root).replace('$REPO_PATH',self.repo_root).replace('\\', '\\\\').replace('/', '\\\\')
                else:
                    self.tf_cli_args = obj.get('tf_cli_args', '').replace('"','').replace('${REPO_PATH}',self.repo_root).replace('$REPO_PATH',self.repo_root)
        except KeyError:
            console.error("  Missing Common Shell Env File: \n          {}\n".format(self.common_shell_file), showTime=False)
            sys.exit(2)
        except(ValueError):
            pass   

    def sanity_check(self):
        """
        Series of sanity Checks performed to ensure what we
        are working with.
        """

        self.var_file_args_list = []
        self.var_file_args = ''

        if os.path.isfile(self.secret_path):
            self.var_file_args_list.append(self.secret_path)

        if self.location == self.repo_root:
            console.error("  You are executing " + self.app_name.upper() + " from the repository root !\n          Please ensure execution from a resurce directory !\n", showTime=False)
            sys.exit(2)

        if not os.path.isfile(self.local_env_file):
            console.error("  No Local Environment Files at this location !\n", showTime=False)
            if not any(File.endswith(".tf") for File in os.listdir(self.location)):
                console.error("  You are executing " + self.app_name.upper() + " from an improper location,\n          Please ensure execution from a resurce directory !\n", showTime=False)
                sys.exit(2)
            else:
                console.error("  Please create:\n          " + self.local_env_file + "\n          and add configuration content if necessary !\n", showTime=False)
                sys.exit(2)
        else:
            self.var_file_args_list.append(self.local_env_file)

        if not os.path.isfile(self.common_env_file):
            console.success("  No Common Environment File available ! Please create:\n            " + self.common_env_file + "\n            and add configuration content if necessary !\n", showTime=False)
            #sys.exit(2)
        else:
            self.var_file_args_list.append(self.common_env_file)

        if not self.region:
            if self.cloud in ['aws', 'azr']:
                console.error("  Specify 'region' in the file: \n          " + self.common_shell_file, showTime=False)
                sys.exit(2)
        
        if self.target_environment_type not in ["region", "site"]:
            console.error("  Specify a valid Target Environment Type (region/site) in the file: \n  " + self.common_shell_file, showTime=False)
            sys.exit(2)

        arg_prefix = '-var-file='
        self.var_file_args = [arg_prefix + item for item in self.var_file_args_list]

    def set_site_configuration(self):
        """
        Parse Data returned by get_deployment_attributes and
        return prefix and module for blue/green, site or
        the default region based deployment.
        """

        if self.target_environment and self.target_environment_type != 'region':
            self.site = self.target_environment
            if self.mode != '':
                self.prefix = "{}-{}-{}".format(self.project, self.target_environment, self.mode)
                self.module = "{}-{}".format(self.resource, self.mode)
            else:
                self.prefix = "{}-{}".format(self.project, self.target_environment)
                self.module = self.resource
        else:
            self.site = ''
            if self.mode != '':
                self.prefix = "{}-{}".format(self.project, self.mode)
                self.module = "{}-{}".format(self.resource, self.mode)
            else:
                self.prefix = self.project
                self.module = self.resource

    def set_backend_configuration(self, backend_type=None):
        """
        Parse Data returned by get_deployment_attributes and
        return bucket, backend_region for deployment.
        """

        self.backend_type = backend_type or self.cloud
        self.backend_region = None
        self.tf_cloud_backend_org = None
     
        if self.backend_type == "aws":
            if self.global_resource == "true" or any(word in self.resource for word in self.global_resources):
                self.bucket_key = "{prefix}/{module}/terraform.tfstate".format(
                    prefix=self.prefix,
                    module=self.module
                )            
            else:
                self.bucket_key = "{prefix}/{region}/{module}/terraform.tfstate".format(
                    prefix=self.prefix,
                    region=self.region,
                    module=self.module
                )

            if self.dr == "true":
                self.bucket = "{}.{}.{}.dr".format(self.bucket_prefix, self.account, self.environment)
                if self.china_deployment == "true":
                    self.backend_region = "cn-northwest-1"
                else:
                    self.backend_region = "us-west-2"
            else:   
                self.bucket = "{}.{}.{}".format(self.bucket_prefix, self.account, self.environment)
                if self.china_deployment == "true":
                    self.backend_region = "cn-north-1"
                else:
                    self.backend_region = "us-east-1"
        elif self.backend_type == "azr":
            if self.global_resource == "true" or any(word in self.resource for word in self.global_resources):    
                self.bucket_key = "{env}/{prefix}/{module}/terraform.tfstate".format(
                    env=self.environment,
                    region=self.region,
                    module=self.module
                )
            else:
                self.bucket_key = "{env}/{prefix}/{region}/{module}/terraform.tfstate".format(
                    env=self.environment,
                    prefix=self.prefix,
                    region=self.region,
                    module=self.module
                )

            self.bucket_prefix = ''.join(char.lower() for char in self.bucket_prefix if char.isalnum())
            if self.dr == "true":
                self.bucket = "{}{}{}dr".format(self.bucket_prefix, self.account, self.environment)
            else:
                self.bucket = "{}{}{}".format(self.bucket_prefix, self.account, self.environment)
            
            if len(self.bucket) > 24:
                console.error("  Storage Account Name exceeds 24 characters.\n  Storage Account: " + self.bucket + "\n  Please provide a shorter name.\n", showTime=False)
                sys.exit(2)

        elif self.backend_type == "tfc" or self.tf_cloud_backend == "true":
            if self.tf_cloud_backend == "simple":
                self.bucket_key = "{env}-{prefix}-{module}".format(
                    env=self.environment,
                    prefix=self.prefix.replace("/","-"),
                    module=self.module.replace("/","-")
                )
            elif self.tf_cloud_backend == "extended":
                if self.global_resource == "true" or any(word in self.resource for word in self.global_resources):
                    self.bucket_key = "{cloud}-{env}-{prefix}-{module}".format(
                        cloud = self.cloud,
                        env=self.environment,
                        prefix=self.prefix.replace("/","-"),
                        module=self.module.replace("/","-")
                    )
                else:
                    self.bucket_key = "{cloud}-{env}-{prefix}-{region}-{module}".format(
                        cloud = self.cloud,
                        env=self.environment,
                        prefix=self.prefix.replace("/","-"),
                        region=self.region.replace("-",""),
                        module=self.module.replace("/","-")
                    )
            else:
                console.error("  Invalid tf_cloud_backend value.\n  Please provide a valid value: simple/extended\n", showTime=False)
                sys.exit(2)
            if self.tf_cloud_org1:
                self.tf_cloud_backend_org = self.tf_cloud_org1
            else:
                self.tf_cloud_backend_org = self.tf_cloud_org2
            self.backend_region = "none"
            self.bucket = "none"
        else:
            self.bucket_key = "none"
            self.backend_region = "none"
            self.bucket = "none"

    def export_environment(self):
        """
        Export the environemt Variables to be used by
        Terraform.
        """
        
        self.my_env = dict(os.environ, 
            **{"TF_VAR_deployment_region": self.region},
            **{"TF_VAR_backend_region": self.backend_region},
            **{"TF_VAR_project": self.project},
            **{"TF_VAR_account": self.account},
            **{"TF_VAR_mode": self.mode},
            **{"TF_VAR_env": self.environment},
            **{"TF_VAR_site": self.site},
            **{"TF_VAR_azrsa": self.bucket},
            **{"TF_VAR_bucket": self.bucket},
            **{"TF_VAR_prefix": self.prefix},
            **{"TF_VAR_china_deployment": self.china_deployment},
            **{"TF_CLI_ARGS": self.tf_cli_args},
            **{"AWS_REGION": self.region},
            **{"AZR_REGION": self.region},
            **{"REPO_PATH": self.repo_root},
            **{"REPO_PREFIX": self.repo_prefix},
            )



import os
from setuptools import setup

with open("README.md", "r") as readme_fd:
    long_description = readme_fd.read()

setup(
    name="tfbuild",
    setup_requires=['setuptools-git-versioning'],
    setuptools_git_versioning={
        "template": "{tag}",
        "dev_template": "{tag}",
        "dirty_template": "{tag}",
        },
    author="Michael Pearson",
    description="TFBuild is a command line tool for Terraform ",
    url="https://github.com/mpearson117/tfbuild",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=['tfbuild'],
    package_dir={"": "src"},
    install_requires=[
        "boto3",
        "colorlog",
        "confuse",
        "GitPython", 
        "jsonpickle", 
        "py_console",
        "PyYAML", 
        "pyhcl",
        "requests" 
        ],
    extras_require={
        'dev:python_version > "3"': ["pytest>=5.0,<6",],
        "dev": {"pytest-cov", "pytest-mock", "codecov"},
    },
    entry_points={"console_scripts": ["tfbuild = tfbuild.cli:main","tfb = tfbuild.cli:main",],},
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Information Technology",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Pre-processors",
        "Topic :: System :: Systems Administration",
    ],
)
